/*
 ==============================================================================================================================================
 Author      : CHEONG SE HANG (dit.dhc@lostcity-studio.com)
 Description : The implementation of Queue.
 Usage       : struct ABC *abc = malloc( sizeof( struct ABC ) );
               abc->a = 2;
               abc->b = 2.2;
               struct Queue *Q = Queue_Init( 1024 );
               Queue_Insert( Q, ( int* )abc );
               ( struct ABC* )Queue_Get( Q ) )->a;
               free( Queue_Pop( Q ) );
               Queue_Destroy( Q );
 ==============================================================================================================================================
 */
#ifndef Queue4SendData_H_
#define Queue4SendData_H_
#include "../Client.h"
/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Structure Definitions
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
struct Queue4SendData {  /* Queue structure. */
    int count;      /* Number of elements in the queue. */
    int front;      /* The position of first element. */
    int rear;       /* The position of next to the last element. */
    int size;       /* Maximum size of queue. */
    struct SendDataStruct qlist[ 4096 ];    /* List of element. */
};




/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Public Functions
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Clear and free all items in the queue.
 * @param Q                 Pointer to the @struct Queue.
 */
void Queue4SendData_Clear ( struct Queue4SendData *Q );


/**
 * Free the the queue and its items.
 * @param Q                 Pointer to the @struct Queue.
 */
void Queue4SendData_Destroy ( struct Queue4SendData *Q );


/**
 * Get the pointer to the first (ahead) item.
 * @param Q                 Pointer to the @struct Queue.
 * @return Returns the pointer to the first (ahead) item.
 */
struct SendDataStruct Queue4SendData_Get ( struct Queue4SendData *Q );


/**
 * Get the number of item in the queue.
 * @param Q                 Pointer to the @struct Queue.
 * @return Returns the number of item in the queue; returns -1 when @Q is NULL.
 */
int Queue4SendData_GetCount ( struct Queue4SendData *Q );


/**
 * Construct a queue.
 * @param size              Maximum sizes of queue.
 * @return Returns the allocated pointer to the queue.
 */
void Queue4SendData_Init ( struct Queue4SendData* q, int size );


/**
 * Insert an item at the end of queue.
 * @param Q                 Pointer to the @struct Queue.
 * @param addr              Pointer to the content which will be added to the queue.
 * @return Returns 0 on success; returns -1 on failure or queue is full.
 */
int Queue4SendData_Insert ( struct Queue4SendData *Q, struct SendDataStruct addr );


/**
 * Delete and retrieve the pointer to the first (ahead) item.
 * WARNING: User must free the returned item in external,
 *          otherwise, it is a memory leak.
 * @param Q                 Pointer to the @struct Queue.
 * @return Returns the pointer to the first (ahead) item.
 */
struct SendDataStruct Queue4SendData_Pop ( struct Queue4SendData *Q );




#endif /* Queue4SendData_H_ */
