/*
 ==============================================================================================================================================
 Author      : CHEONG SE HANG (dit.dhc@lostcity-studio.com)
 Description : The extension to time library.
 ==============================================================================================================================================
 */
#ifndef TIME_H_
#define TIME_H_
#include "Type.h"
#include <unistd.h>
/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Structure Definitions
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
typedef unsigned long long TIMER;           /* Data type used in time library. */




/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Public Functions
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Make program sleep in million seconds (1s=1000ms).
 * @param x         Number of million seconds to sleep.
 */
#define msleep( x ) usleep( x * 1000 )


/**
 * Get the time-stamp since 1970-01-01 00:00 to million seconds.
 * @return Returns the time-stamp in million seconds.
 */
TIMER time2MS ();


/**
 * Get the time-stamp since 1970-01-01 00:00 to seconds.
 * @return Returns the time-stamp in seconds.
 */
INT32 time2Sec ();


/**
 * Convert time to string (in human readable format).
 * @param out       Pointer to output string.
 */
void time2Str ( char* out );


/**
 * Check timeout.
 * @param clk       Time-stamp in million seconds.
 * @param ms        Number of million seconds.
 * @return Returns TRUE if @clk with @ms is timeout; otherwise, returns FALSE.
 */
BOOL timeout ( TIMER clk, UINT32 ms );




#endif /* TIME_H_ */
